#ifndef _NETPORT_H
#define _NETPORT_H

/* network port for the server to listen on/client to connect to */
#define NETPORT_PORT 21433

/* a string sent by the client at initial connection */
#define NETPORT_HANDSHAKE "NETPORT0"

/* client stub functions */
void netport_end(void);
void netport_nop(void);
void outw (unsigned short int value, unsigned short int port);
unsigned short int inw (unsigned short int port);
void outb (unsigned char value, unsigned short int port);
unsigned char inb (unsigned short int port);
int ioperm(unsigned long from, unsigned long num, int turn_on);

enum netport_commands {
    cmd_nop     =   0,
    cmd_end     =   1,
    cmd_outw    =   2,
    cmd_inw     =   3,
    cmd_outb    =   4,
    cmd_inb     =   5,
    cmd_ioperm  =   6
};

/*

netport network protocol (over TCP)

Upon initial connection the client sends a string "NETPORT0" to the server.
The client then proceeds to send a series of commands to the server, which
responds to certain types of commands. The connection ends if either the client
sends a cmd_end command or the client or server terminate the connection.

commands:

byte 0 is always the number of the command
all integers are unsigned and in network byte order

cmd_nop:
    no body
    1 byte response
    response byte 0: a literal 0 byte
    does nothing, for pinging purposes

cmd_end:
    no body
    no response
    close connection

cmd_outw:
    4 byte body
    bytes 1-2: port number
    bytes 3-4: value to write to port
    no response
    write a two byte value to a port

cmd_inw:
    2 byte body
    bytes 1-2: port number
    2 byte response
    response bytes 0-1: value read from port
    read a two byte value from a port

cmd_outb:
    3 byte body
    bytes 1-2: port number
    byte 3: value to write to port
    no response
    write a one byte value to a port

cmd_inb:
    2 byte body
    bytes 1-2: port number
    1 byte response
    response byte 0: value read from port
    read a one byte value from a port

cmd_ioperm:
    9 byte body
    bytes 1-4: starting port number
    bytes 5-8: number of port addresses to set permissions on
    byte 9: 0 to disable permission, 1 to enable
    5 byte response
    byte 0: 0 on success, 1 on failure
    bytes 1-4: errno of failure (undefined but still present on success)

    set permissions on a series of port addresses

    Should this be necessary? Couldn't we just
    set this as the requests come in?

*/

/* handy utilities */
int read_or_fail(int fd, unsigned char buf[], int length);
int write_or_fail(int fd, const unsigned char buf[], int length);

#endif
